#!/usr/bin/python
#
# findDupIP.py
# version 1.1
#
#
# Look for duplicate IP addresses in all the host objects.
# Written by: Check Point Software Technologies inc. 
# December 2015
# Updated: December 2017 for R80.10 API version
#


# A package for reading passwords without displaying them on the console.
from __future__ import print_function

import getpass
import sys, os
import json
import argparse

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

# lib is a library that handles the communication with the Check Point management server.
from lib import APIClient, APIClientArgs

def example_inventory():
    return {
        'group': {
            'hosts': ['192.168.28.71', '192.168.28.72'],
            'vars': {
                'ansible_ssh_user': 'vagrant',
                'ansible_ssh_private_key_file':
                    '~/.vagrant.d/insecure_private_key',
                'example_variable': 'value'
            }
        },
        '_meta': {
            'hostvars': {
                '192.168.28.71': {
                    'host_specific_var': 'foo'
                },
                '192.168.28.72': {
                    'host_specific_var': 'bar'
                }
            }
         }
    }

# Read the command line args passed to the script.
def read_cli_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--list', action = 'store_true')
    parser.add_argument('--host', action = 'store')
    args = parser.parse_args()


def main():
   
    obj_dictionary = {}

    #print("We start here. ")
    #example_inventory()
    #sys.exit()   
  
    #getting details from the user
    #api_server = raw_input("Enter server IP address or hostname:")
    #username = raw_input("Enter username: ")
    #if sys.stdin.isatty():
    #    a = 1
		#password = getpass.getpass("Enter password: ")
    #else:
    #    print("Attention! Your password will be shown on the screen!")
    #    password = raw_input("Enter password: ")
    
    api_server = "192.168.168.50"
    username = "admin"
    password = "qwe123"
    print("Api_server: "+api_server)
    #sys.exit() 	
       			
    client_args = APIClientArgs(server=api_server)

    with APIClient(client_args) as client:

        # create debug file. The debug file will hold all the communication between the python script and
        # Check Point's management server.
        client.debug_file = "api_calls.json"

        # The API client, would look for the server's certificate SHA1 fingerprint in a file.
        # If the fingerprint is not found on the file, it will ask the user if he accepts the server's fingerprint.
        # In case the user does not accept the fingerprint, exit the program.
        if client.check_fingerprint() is False:
            print("Could not get the server's fingerprint - Check connectivity with the server.")
            exit(1)

        # login to server:
        login_res = client.login(username, password)
        if login_res.success is False:
            print("Login failed: {}".format(login_res.error_message))
            exit(1)

        print("Login to the CP management succeeded.")
		
		# get all domains 
        print("we collecting all existing domains.")
        show_domains_res = client.api_query("show-domains", "full")
        if show_domains_res.success is False:
            print("Failed to get the list of domains: {}".format(show_domains_res.error_message))
            exit(1)
        # just list domains
        for domain in show_domains_res.data:
            print("domain: ") 
            print(domain)
            
			#print out all the keys
            	
			# print key n values from the whole response
            print("### ### ### ###") 
            print("Print key and values here") 
            for k, v in domain.items():
                print("--- ")			
                print("Key : {0}, Value : {1}".format(k, v))	
            
            domainName=domain.get("name")
            
            # get domain ip
            # the section "server" contains the domain ip 	
			# NOTE: that section is corrupted:
			#       the value of section is an object with a key containing the actual value, the value doesn't exit
		    # get the section "server"
            serverSection = domain.get("servers")
            print("server section: ") 
            print(serverSection) 			
            for key in serverSection:
                print("list key: ")
                print(key)
                domainIP = key["ipv4-address"]
			
            print("domainName: "+domainName)
            print("domainIP: "+domainIP)
							
            continue
			
            sys.exit()
			
			
		    # get info from domain section 
			#Key : domain, Value : {u'domain-type': u'mds', u'uid': #u'a0eebc99-afed-4ef8-bb6d-fedfedfedfed', u'name': u'System #Data'}
            #domainSection = domain.get("domain")
            #print("domainSection: ")     
            #print(domainSection["uid"])
				
            sys.exit()	
			
			
            domainName = domain.get("name")
            print("domainName: "+domainName) 
            domainIPList = domain.get("servers")
            
            print("domainIPList type:")
            print(type(domainIPList))
            print("domainIPList:")
            print(domainIPList)
			
			#print out all the keys
            for c in domainIPList:
                print("list item: ")
                print(c)
                print("Type of c:")
                print(type(c))
                print("the ipv4-address:")
                print(c["ipv4-address"]) 				
			
			#print keys and values
            #for k, v in domainIPList.items():
            #    print("Code : {0}, Value : {1}".format(k, v))
			
			
            #ip = domainIPList["ipv4-address"]
            #print("ip: "+ip) 
 			
			
            room_num = {'john': 425, 'tom': 212}
            print("room_num:") 
            print(room_num)
            print("room_num type:")
            print(type(domainIPList))
            #print(room_num['john'])
			
			
			
			
			#ip = domainIPList['ipv4-address']
            
			#for item in ['apple','banana','mango']:
            #    print("I like",fruit)
			#domainIPList_json = json.loads(domainIPList)
            #print(domain) 
            #domainIPList_json = json.loads(domainIPList)
            #print("domainIP: ") 
            #print(domainIPList_json) 
            #ip = domainIPList.get("ipv4-address")
            #print("ip: "+ip) 
 			#json.dumps(domainIPList.as_dict())
			
        sys.exit()	
		
		# logout
		# ToDo implement this function in the lib
        #logout_res = client.logout()
		#if logout_res.success is False:
        #    print("Logout failed: {}".format(logout_res.error_message))
        #    exit(1)
        #print("Logout to the CP management succeeded.")
		
		
		
		
		
		
		

        # get gateways from each domain		
        for domain in show_domains_res.data:
            domainName = domain.get("name")
        	
			# login to MDS to domain
     	    login_res = client.login(username, password, False, domainName)
            if login_res.success is False:
                print("Login failed: {}".format(login_res.error_message))
                exit(1)
            print("Login to the CP management to the domain "+domainName+" succeeded.")
		    
			# get gateways from that domain
            show_gateways_res = client.api_query("show gateways-and-servers", "standard")
            if show_gateways_res.success is False:
                print("Failed to get the list of gateways: {}".format(show_gateways_res.error_message))
                exit(1)
            # just list gateways
        
		
		
        sys.exit()
        
        if show_gateways_res.success is False:
            print("Failed to get the list of gateways: {}".format(show_gateways_res.error_message))
            exit(1)        
            for gw in show_gateways_res.data:
                gwName = gw.get("name")
                print("gwName: "+gwName) 		
		
            			
        # show hosts
        #print("Processing. Please wait...")
        #show_hosts_res = client.api_query("show-hosts", "standard")
        #if show_hosts_res.success is False:
        #    print("Failed to get the list of all host objects: {}".format(show_hosts_res.error_message))
        #    exit(1)

    # obj_dictionary - for a given IP address, get an array of hosts (name, unique-ID) that use this IP address.
    obj_dictionary = {}

    # dup_ip_set - a collection of the duplicate IP addresses in all the host objects.
    dup_ip_set = set()

    for host in show_hosts_res.data:
        ipaddr = host.get("ipv4-address")
        if ipaddr is None:
            print(host["name"] + " has no IPv4 address. Skipping...")
            continue
        host_data = {"name": host["name"], "uid": host["uid"]}
        if ipaddr in obj_dictionary:
            dup_ip_set.add(ipaddr)
            obj_dictionary[ipaddr] += [host_data]  # '+=' modifies the list in place
        else:
            obj_dictionary[ipaddr] = [host_data]

    # print list of duplicate IP addresses to the console
    print("\n")
    print("List of Duplicate IP addresses: ")
    print("------------------------------- \n")

    if len(dup_ip_set) == 0:
        print("No hosts with duplicate IP addresses")

    # for every duplicate ip - print hosts with that ip:
    for dup_ip in dup_ip_set:
        print("\nIP Address: " + dup_ip + "")
        print("----------------------------------")

        for obj in obj_dictionary[dup_ip]:
            print(obj["name"] + " (" + obj["uid"] + ")")


if __name__ == "__main__":
    main()
