#!/usr/bin/python
#
# (c) 2019 Check Point Software Technologies
#
# Script performs following:
#   - Connects to the MDS via MGMT API
#   - Collects the data of all gateways in all CMAs
#   - Put the data into list of gateway objects
#   - Creates ansible inventories for following GAiA configuration actions:
#       - set ntp
#       - set dns
#       - set/delete user
#       - copy user public key
#
# The connection to managament server via CP MGMT API is based on the code from following project:
# https://github.com/CheckPointSW/cp_mgmt_api_python_sdk
#
#
# VER   DATE            WHO                     WHAT
#------------------------------------------------------------------------------
# v1.0 -               Yevgeniy Yeryomin       Creation of the script

# A package for reading passwords without displaying them on the console.
from __future__ import print_function

import getpass
import sys, os
import json
import argparse
import csv
import os.path as path
import subprocess
#
import Helper

# Get the current path
#ypath="/home/yevgeniy/ansible/GatewaysManagementAutomated/"
mypath =  path.abspath(path.join(__file__ ,"../.."))
print("Projects path: "+mypath)
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), mypath+'/scripts/inventoryGenerators/')))
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), mypath+'/scripts/classes')))
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '../lib/cp_mgmt_api_python_sdk')))
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), mypath+'/scripts/')))


print("sys.path")
print(sys.path)

import Gateway
import getDNSData
import getNTPData
import getUserGroupData


### SCRIPT CONFIGURATION
dnsConfigFile=mypath+"/servicesConfig/dns.conf"
ntpConfigFile=mypath+"/servicesConfig/ntp.conf"
userGroupsConfigFile=mypath+"/servicesConfig/userGroups.conf"

ansibleDNSInventoryFile=mypath+"/vars/gateways-dns-dyn.yml"
ansibleNTPInventoryFile=mypath+"/vars/gateways-ntp-dyn.yml"
ansibleUserGroupsInventoryFile=mypath+"/vars/gateways-userGroups-dyn.yml"
ansibleUserGroupsCopyKeysInventoryFile=mypath+"/vars/ansibleHosts_copyKeys_dyn.yml"
#fingerprintsFile=mypath+"/scripts/fingerprints_.txt"
ansibleHosts=mypath+"/vars/ansibleHosts.yml"

ansibleHostsConfig={
  "mgmt_server": "",
  "mgmt_user": "",
  "mgmt_password": "",
  #"ansible_python_interpreter": "",
  "ansible_user": "",
  "ansible_ssh_pass": "",
  "local_key_path": "",
  "remote_key_path": "",
  "remote_key_fileName": ""
}


# lib is a library that handles the communication with the Check Point management server.
from lib import APIClient, APIClientArgs

def getFingerprint(_gwIP):
    fingerprint=""
    #fingerprint=os.popen("openssl s_client -connect 192.168.168.121:443 < /dev/null 2>/dev/null | openssl x509 -fingerprint -noout -in /dev/stdin")
    command="openssl s_client -connect "+_gwIP+":443 < /dev/null 2>/dev/null | openssl x509 -fingerprint -noout -in /dev/stdin"
    tmpFile="tmp.txt"
    os.system(command+" > "+tmpFile)
    output=open(tmpFile, 'r').read()
    os.remove(tmpFile)

    outputArr=output.split("=")
    if len(outputArr)<2:
        print("ERROR. Fingerprint for "+_gwIP+" could not be retrieved.")
        print("We stop here.")
        ### STOP
        #exit()
    else:
        fingerprint=outputArr[1].strip()
    print("fingerprint: ")
    print("---"+fingerprint+"---")
    return fingerprint



def setConfigurationVars():
    # Read ansible hosts config file and get the login vars
    print("--- ---")
    print("Get ansible host configuration from inventory file: "+ansibleHosts)
    with open(ansibleHosts, "r") as ins:
        for line in ins:
            line=line.strip()
            #print("Line: "+line)
            if line.startswith("#"):
                continue
            if line.startswith("false"):
                continue
            # iterate trough all dict keys
            for key in ansibleHostsConfig:
                if line.startswith(key+"=") or line.startswith(key+" ="):
                    configError=False
                    #print("Value found:")
                    #print(" key:"+key)
                    #print(" line:"+line)
                    lineStr=line.split("=")
                    #print("len(lineStr):"+str(len(lineStr)))
                    if len(lineStr)<2:
                        configError=True
                    value=lineStr[1].strip()
                    if value=="":
                        configError=True
                    if configError:
                        # !!! Error we stop here.
                        print("ERROR: Error in following ansible config file: "+ansibleHosts)
                        print("Please check the configured for the key: "+key)
                        exit()
                    #print(" value: ---"+value+"--")
                    ansibleHostsConfig[key]=value

    # Verify all values are filled
    for key, value in ansibleHostsConfig.items():
        if key=="mgmt_password":
            print(key+": ******")
        else:
            print(key+": "+value)
        if value=="":
            print("ERROR: Error in following ansible config file: "+ansibleHosts)
            print("Please check the configured for the key: "+key)
            exit()

def getDictFromCSV():
    reader = csv.DictReader(open(variables_file, 'rb'))
    dict_list = []
    for line in reader:
        dict_list.append(line)
    return dict_list

def example_inventory():
    # an example for dynamice inventory
    return {
        'group': {
            'hosts': ['192.168.28.71', '192.168.28.72'],
            'vars': {
                'ansible_ssh_user': 'vagrant',
                'ansible_ssh_private_key_file':
                    '~/.vagrant.d/insecure_private_key',
                'example_variable': 'value'
            }
        },
        '_meta': {
            'hostvars': {
                '192.168.28.71': {
                    'host_specific_var': 'foo'
                },
                '192.168.28.72': {
                    'host_specific_var': 'bar'
                }
            }
         }
    }

# Read the command line args passed to the script.
def read_cli_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--list', action = 'store_true')
    parser.add_argument('--host', action = 'store')
    args = parser.parse_args()

#Depricated
def getDictFromCSV(_fingerprintsFile):
    with open(_fingerprintsFile) as fpFile:
        fingerprintDict = dict(filter(None, csv.reader(fpFile)))
    #for k, v in fingerprintDict.items():
    #    print("Code : {0}, Value : {1}".format(k, v))
    return fingerprintDict

def getDomainsDict(_show_domains_res):
    domainsDict={}
    for domain in _show_domains_res.data:
        domainName=domain.get("name")
        serversSection=domain.get("servers")
        #print("         ")
        #print("         ")
        #print("All key and values of domain")
        #for gwK, gwV in domain.items():
        #    print("--- ")
        #    print("Key : {0}, Value : {1}".format(gwK, gwV))
        #print(" ")
        #print("Server section: ")
        #print(serversSection)
        for server in serversSection:
            #print("list key: ")
            #print(server)
            domainIP = server["ipv4-address"]

        #print("domainName: "+domainName)
        #print("domainIP: "+domainIP)
        domainsDict[domainName] = domainIP

    # Print the domainDict
    for dK, dV in domainsDict.items():
        print("{0:<12}: {1:<12}".format(dK, dV))

    return domainsDict

def getGwList():

    domainsDict={}
    gwObjsList=[]
    #client_args = APIClientArgs(server=api_server)
    client_args = APIClientArgs(server=ansibleHostsConfig["mgmt_server"])


    # Put data from csv to dict
    # DEPRICATED: the fingerprints are determined via openssl command
    #fingerprintDict=getDictFromCSV(fingerprintsFile)

    with APIClient(client_args) as client:

        # create debug file. The debug file will hold all the communication between the python script and
        # Check Point's management server.
        client.debug_file = "api_calls.json"

        # The API client, would look for the server's certificate SHA1 fingerprint in a file.
        # If the fingerprint is not found on the file, it will ask the user if he accepts the server's fingerprint.
        # In case the user does not accept the fingerprint, exit the program.
        if client.check_fingerprint() is False:
            print("Could not get the server's fingerprint - Check connectivity with the server.")
            exit(1)

        # Login to server:
        print("--- ---")
        ###login_res = client.login(username, password)
        login_res = client.login(ansibleHostsConfig["mgmt_user"], ansibleHostsConfig["mgmt_password"])
        if login_res.success is False:
            print("Login failed: {}".format(login_res.error_message))
            exit(1)
        print("Login to the CP management succeeded.")


		# Get all domains and put them into a dict
        print("--- ---")
        print("Get all existing domains.")
        show_domains_res = client.api_query("show-domains", "full")
        if show_domains_res.success is False:
            print("Failed to get the list of domains: {}".format(show_domains_res.error_message))
            exit(1)
        # Get the domains dict [domainName; domainIP]
        domainsDict=getDomainsDict(show_domains_res)


        # Get the list of gateways for each domain
        print("--- ---")
        print("Get the list of gateways for each domain")
        for domainName, cmaIP in domainsDict.items():
            print("--- ")
            # Login to MDS domain:
            print("Login to the MDS to the following domain:")
            print(" domainName : {0}, cmaIP : {1}".format(domainName, cmaIP))
            #login_res = client.login(username, password, False, domainName)
            login_res = client.login(ansibleHostsConfig["mgmt_user"], ansibleHostsConfig["mgmt_password"], False, domainName)
            if login_res.success is False:
                print("Login failed: {}".format(login_res.error_message))
                exit(1)
            print("Login succeeded.")

            print("Collect all required gateways information")
            show_gateways_res = client.api_query("show-gateways-and-servers", "full")
            if show_gateways_res.success is False:
                print("Failed to get the list of gateways: {}".format(show_gateways_res.error_message))
                exit(1)

            # Proceed on each gateway
            for gateway in show_gateways_res.data:

                # Filter for items of interests
                if str(gateway.get("type"))!="simple-gateway":
                    continue

                # Put the gateway information into list
                gatewayObj=Gateway.Gateway()
                gatewayObj.gw_name=str(gateway.get("name")).strip()
                gatewayObj.gw_ip=str(gateway.get("ipv4-address")).strip()
                gatewayObj.gw_version=str(gateway.get("version")).strip()
                gatewayObj.gw_platform=str(gateway.get("hardware")).strip()
                gatewayObj.gw_os=str(gateway.get("operating-system")).strip()
                gatewayObj.gw_type=str(gateway.get("type")).strip()
                gatewayObj.gw_sicStatus=str(gateway.get("sic-status")).strip()
                gatewayObj.gw_cma=domainName

                # SKIP gwObject considered to be inactive
                if gatewayObj.gw_sicStatus!="communicating":
                    continue

                # Get finderprint
                gatewayObj.gw_fingerprint=getFingerprint(gatewayObj.gw_ip)

                gwObjsList.append(gatewayObj)

            # Logouts
            print("---")
            print("Logout from the MDS domain: "+domainName+" "+cmaIP)
            client.api_call("logout", {})
            print("Logout successeded.")

        # Print all collected gateways
        print("---")
        print("All collected gateways:")
        i=0
        for gwObj in gwObjsList:
            i=i+1
            if i==1:
                print(gwObj.toStrHeaderGwFull())
                print(gwObj.toStrGwFull())
            else:
                print(gwObj.toStrGwFull())



    return gwObjsList


def main():

    # Get the configuration
    setConfigurationVars()

    ### Get gateways from MDS (via CP MGMT API)
    print("--- ---")
    print("Get gateways from MDS")
    gwObjsList=getGwList()

    ### DNS
    # Get DNS config objects and create DNS config for all gateways
    # DNS config from /configs/dns.conf is assigned to each gateway according to attributes (ip, type, os, platform)
    print("--- ---")
    print("Get DNS config for gateways")
    getDNSData.getGwDNSConfList(gwObjsList, ansibleDNSInventoryFile, dnsConfigFile)
    #
    # ### NTP
    # ### Get NTP config objects and create NTP config for all gateways
    print("--- ---")
    print("Get NTP config for gateways")
    getNTPData.getGwNTPConfList(gwObjsList, ansibleNTPInventoryFile, ntpConfigFile)

    # ### User configuration
    # ### add user
    print("--- ---")
    print("Get users to be configured for gateways")
    getUserGroupData.getGwUserGroupConfList(gwObjsList, ansibleUserGroupsInventoryFile, ansibleUserGroupsCopyKeysInventoryFile, userGroupsConfigFile, ansibleHostsConfig)


if __name__ == "__main__":
    main()
